//   Copyright 2012 Georg-August-Universität Göttingen, Germany
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package de.ugoe.cs.autoquest.usability.evaluation.rule.evaluator;

import static de.ugoe.cs.autoquest.usability.evaluation.rule.set.RulesetFactory.textInputUsabiliyRuleset;
import static de.ugoe.cs.autoquest.usability.testutil.FestConditionUtil.absent;
import static de.ugoe.cs.autoquest.usability.testutil.FestConditionUtil.highRecommendationSeverityLevel;
import static de.ugoe.cs.autoquest.usability.testutil.FestConditionUtil.infoRecommendationSeverityLevel;
import static de.ugoe.cs.autoquest.usability.testutil.FestConditionUtil.lowRecommendationSeverityLevel;
import static de.ugoe.cs.autoquest.usability.testutil.FestConditionUtil.mediumRecommendationSeverityLevel;
import static de.ugoe.cs.autoquest.usability.testutil.FestConditionUtil.present;
import static org.fest.assertions.api.Assertions.assertThat;

import java.util.EnumSet;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.google.common.base.Optional;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;

import de.ugoe.cs.autoquest.tasktrees.treeifc.ITaskTree;
import de.ugoe.cs.autoquest.usability.evaluation.result.UsabilityDefect;
import de.ugoe.cs.autoquest.usability.evaluation.rule.set.UsabilityRule;
import de.ugoe.cs.autoquest.usability.tasktree.filter.FilterStatisticCache;

/**
 * <p>
 * TODO comment
 * </p>
 * 
 * @author Alexander Deicke
 */
public class NoLetterOrDigitTextInputsEvaluatorTest extends AbstractUsabilityEvaluationTC {

    private UsabilityRule ruleUnderTest;

    @Before
    public void initRuleUnderTest() {
        EnumSet<? extends UsabilityRule> ruleset = textInputUsabiliyRuleset().evaluationRules();
        this.ruleUnderTest = Iterables.find(ruleset, new Predicate<UsabilityRule>() {

            public boolean apply(UsabilityRule usabilityRule) {
                return usabilityRule.ruleIdentifier().equals("TEXT_FIELD_NO_LETTER_OR_DIGIT_RATIO");
            }

        });
    }

    @After
    public void clearFilterStatisticCache() {
        FilterStatisticCache.instance().clear();
    }

    @Test
    public void should_return_no_recommendation() {
        // Given
        String spec = "Sequence {" + "  TextInput () {}" + "}";
        ITaskTree taskTree = createTaskTree(spec);
        // When
        Optional<UsabilityDefect> recommendation =
            new NoLetterOrDigitTextInputsEvaluator(this.ruleUnderTest, taskTree).evaluationResult();
        // Then
        assertThat(recommendation).is(absent());
    }

    @Test
    public void should_return_recommendation_with_info_severity_level() {
        // Given
        String spec =
            "Sequence {"
                + "  TextInput (1234567890123456789012345678901234567890123456789_01234567890"
                + "12345678901234567890123456789012345) {}" + "}";
        ITaskTree taskTree = createTaskTree(spec);
        // When
        Optional<UsabilityDefect> recommendation =
            new NoLetterOrDigitTextInputsEvaluator(this.ruleUnderTest, taskTree).evaluationResult();
        // Then
        assertThat(recommendation).is(present()).has(infoRecommendationSeverityLevel());
    }

    @Test
    public void should_return_recommendation_with_low_severity_level() {
        // Given
        String spec =
            "Sequence {" + "  TextInput (123456789012345678901234567890_123456789012345) {}" + "}";
        ITaskTree taskTree = createTaskTree(spec);
        // When
        Optional<UsabilityDefect> recommendation =
            new NoLetterOrDigitTextInputsEvaluator(this.ruleUnderTest, taskTree).evaluationResult();
        // Then
        assertThat(recommendation).is(present()).has(lowRecommendationSeverityLevel());
    }

    @Test
    public void should_return_recommendation_with_medium_severity_level() {
        // Given
        String spec = "Sequence {" + "  TextInput (12345_6789012345) {}" + "}";
        ITaskTree taskTree = createTaskTree(spec);
        // When
        Optional<UsabilityDefect> recommendation =
            new NoLetterOrDigitTextInputsEvaluator(this.ruleUnderTest, taskTree).evaluationResult();
        // Then
        assertThat(recommendation).is(present()).has(mediumRecommendationSeverityLevel());
    }

    @Test
    public void should_return_recommendation_with_high_severity_level() {
        // Given
        String spec = "Sequence {" + "  TextInput (_a_b_c_) {}" + "}";
        ITaskTree taskTree = createTaskTree(spec);
        // When
        Optional<UsabilityDefect> recommendation =
            new NoLetterOrDigitTextInputsEvaluator(this.ruleUnderTest, taskTree).evaluationResult();
        // Then
        assertThat(recommendation).is(present()).has(highRecommendationSeverityLevel());
    }
}
